function d = otdist( t0,t1,M,N,rel )
%OTDIST Compute L2 optimal transport distance between 
%               two circular ADSN models.
%
%   d = otdist( t0,t1,M,N,rel ) computes the L2 optimal transport 
%   distance between the color circular ADSN models of size [M,N] 
%   corresponding to the kernels t0 and t1.
%   If the optional argument rel is set to 1, then the distance
%   is renormalized by the modulus of the DFT of t0.
%
%   NB : Inputs t0 and t1 can have multiple channels.
%
%   WARNING ! 
%     - The mean coefficient is ignored in the comparison.
%     -  It is assumed that the ADSN model on [M,N] is obtained
%       by extending the shot by zero-padding.
%
%   Author : Arthur Leclaire
%   v. 1.0 (03/2014) : first public version
%   v. 1.1 (10/2015) : 0-frequency must be included in the sum.

    [m0,n0,D] = size(t0);
    [m1,n1,C] = size(t1);
   
    if (C~=D)
        error('t0 and t1 must have the same number of channels');
    end
    
    if ((M<m0)||(N<n0)||(M<m1)||(N<n1))
        error('[M,N] must be greater than the dimensions of t and tref')
    end
    
    d0 = 0;
    d1 = 0;
    sp = zeros(M,N,C);
    for c=1:C
        bigt0 = [t0(:,:,c) zeros(m0,N-n0); zeros(M-m0,N)];
        ft0 = fft2(bigt0);
        bigt1 = [t1(:,:,c) zeros(m1,N-n1); zeros(M-m1,N)];
        ft1 = fft2(bigt1);
        d0 = d0 + sum(sum(abs(ft0).^2));
        d1 = d1 + sum(sum(abs(ft1).^2));
        sp(:,:,c) = conj(ft0).*ft1;
    end
    d = sqrt(d0 + d1 - 2*sum(sum(abs(sum(sp,3)))));

    if ((nargin>4)&&(rel==1))
        d = d/sqrt(d0);
    else
        d = d/(M*N);
    end
    
end

