function [t,evodot,evoerr] = so_texton_rgb(u,env,options)
%SO_TEXTON_RGB Computes a Synthesis-Oriented Texton of a color texture.
%
%   [t,evodot,evoerr] = so_texton_rgb(u,env,options)
%   computes the solution of an optimization problem including
%   	- an envelope constraint,
%       - a Fourier modulus constaint,
%   by alternating projections on those two constraints.
%
%   u : input texture
%   env : input envelope
%   t : output synthesis-oriented texton
%
%   Optional outputs :
%   evodot : evolution of the L2 optimal transport distance to
%       the circular gaussian model associated to u.
%   evoerr : relative l2 changes between two iterates
%
%   Optional inputs :
%   [options.nmax : maximum number of iterations ]
%   [options.err (default 1e-3) : the algorithm stops as soon as the
%        relative l2 error between two iterates is < options.err ]
%   [options.init: initialize with
%       (0, default) a white noise
%       (1) luminance/canonical texton
%       (2) a piece of u
%       (3) a customized choice given in input.initcustom ]
%   [options.visu : allows dynamical visualization of the process
%       The texton visualization is shifted if options.visu==0 .]
%   [options.intexton : allows call to so_texton_rgb with input u equal
%       to a texton; this avoids the normalization step.]
%   [options.nonverbose : disable console output]
%
%   NB : The envelope can take the value Inf. In particular, if for 
%       a set A of logical indices one has env(A)==Inf and env(~A)==0,
%       then the envelope constraint is actually a support constraint.
%
%   The Synthesis-Oriented Texton is described in the paper
%       "A Texton for Fast and Flexible Gaussian Texture Synthesis."
%        (Bruno Galerne, Arthur Leclaire, Lionel Moisan), 
%        proceedings of the European Signal Processing Conference
%        (Eusipco), 2014.
%   If you use this code for further work, please be sure to make
%   reference to this paper.
%
%   Author : Arthur Leclaire
%   v. 1.0 (03/2014) : first public version
%   v. 1.1 (10/2015) : merged the RGB and graylevel cases

if isfield(options,'nmax')
    nmax = options.nmax;
    nolimit = 0;
else
    nmax = 1;
    nolimit = 1;
end

if ~isfield(options,'err')
    options.err = 1e-3;
end

if ~isfield(options,'init')
    options.init = 0;
end

[M,N,C] = size(u);

srcsup = sqrt(sum(sum(abs(env(:,:,1)>0))));

mu = repmat(mean(mean(u,2)),[M N 1]);

if ~isfield(options,'intexton')
    u = 1/sqrt(M*N)*(u-mu);
end

fu = fft2(u);

t = zeros(size(u));

switch options.init
    case 1  % Luminance/canonical texton
        if C==3
        ph = angle(1/3*fu(:,:,1)+1/2*fu(:,:,2)+1/6*fu(:,:,3));
        for c=1:C
            t(:,:,c) = real(ifft2(fu(:,:,c).*exp(-1i*ph)));
        end
        else
            t = real(ifft2(abs(fu)));
        end
    case 2  % Original texture
        for c=1:C
            t(:,:,c) = fftshift(u(:,:,c));
        end
    case 3  % Custom
        t = options.initcustom;
    otherwise  % white noise (default)
        t = randn(M,N,C);
end

% Computes the Fourier transform of t for the first iteration.
ft = fft2(t);
% Open window for visualization
if isfield(options,'visu')
    figure;
end

niter = 0;
err = 1;
while ((err>options.err)&&((niter<nmax)||nolimit))
    niter = niter+1;
    tp = t;
    % Projection of t on the Gaussian model defined by u
    sp = sum(ft.*conj(fu),3);
    sp = repmat(exp(1i*angle(sp)),[1,1,C]);
    ft = sp.*fu;
    t = real(ifft2(ft));
    % Envelope projection
    t(t>env) = env(t>env);
    t(t<-env) = -env(t<-env);
    % Compute the error to the Gaussian model
    %   (the mean coefficients are ignored)
    ft = fft2(t);
    sp = zeros(M,N,C);
    d0 = 0; d1 = 0;
    for c=1:C
        d0 = d0 + sum(sum(abs(fu(:,:,c)).^2));
        d1 = d1 + sum(sum(abs(ft(:,:,c)).^2));
        sp(:,:,c) = conj(fu(:,:,c)).*ft(:,:,c);
    end
    gme = sqrt(d0 + d1 - 2*sum(sum(abs(sum(sp,3)))))/sqrt(d0);    
    
    % Visualization
    if isfield(options,'visu')
        if options.visu~=0
            imshow(mu+srcsup*t)
        else
            imshow(circshift(mu+srcsup*t,[floor(M/2) floor(N/2)]))
        end
        title(['RME = ' num2str(gme)])
        drawnow
    end
    
    % L2 relative difference between current and last iterates
    err = sqrt(sum(sum(sum((tp-t).^2))))/sqrt(sum(sum(sum(t.^2))));
    if ~isfield(options,'nonverbose')
        disp(['Iteration ' num2str(niter) ...
        ', RME = ' num2str(gme) ...
        ', l2 relative change = ' num2str(err)]);
    end
    
    evodot(niter) = gme;
    evoerr(niter) = err;
end

end
